import re
import xml.etree.ElementTree as ET
from optparse import OptionParser
from solution_verifier import parse_xml_answer

def is_x(var):
    return var.startswith('x')
def is_y(var):
    return var.startswith('y')

def parse_x(name):
    return re.sub('(_)(\d)', lambda m: m.group(1) + "&" + m.group(2), name).replace('&', ' ').split()[1:]

def parse_y(name):
    return name.replace('&', ' ').replace('_', ' ').split()[1:]

def get_structures(answer):
    v = {}
    for var, val in answer:
        if is_x(var) and val == 1:
            node, first, second = parse_x(var)
            if node not in v:
                v[node] = []
            v[node].append((first, second))
        elif is_y(var) and val == 0:
            node, name = parse_y(var)
            first = name + '__'
            second = name + '_'
            if node not in v:
                v[node] = []
            v[node].append((first, second))
    return v

def edges_to_graph(edges):
    graph = {}
    for first, second in edges:
        if first not in graph:
            graph[first] = []
        graph[first].append(second)
        
        if second not in graph:
            graph[second] = []
        graph[second].append(first)
    return graph

def restore_names(path, ind_names, z_vars):
	new_path = []
	for gene in path:
		if gene.startswith('-'):
			start = 1
		else:
			start = 0
		 
		if gene[start:] in z_vars:
			new_gene = gene[:start] + z_vars[gene[start:]]
		else:
			new_gene = gene

		dot_pos = new_gene.find('.')
		index = new_gene[start:dot_pos]
		if start != 0:
			start = '-'
		else:
			start = ''

		new_path.append(start + ind_names[index] + new_gene[dot_pos:])
	#print '\npath before restore: ', path
	#print z_vars
	#print 'path after restore: \n', new_path
	return new_path

def build_path(graph, ind_names, structs_file, z_vars):
    used = dict((key.replace('_',''), False) for key in graph.keys())
    
    for vertex in used.keys():
        if not used[vertex]:
            path = [vertex, ]
            #print path
            used[vertex] = True
            while True:
                if path[0].startswith('-'):
                    first = path[0][1:] + '__'
                else:
                    first = path[0] + '_'
                #print 'Searching gene before ', first
                inserted_first = False
                for v in graph[first]:
                    #print 'Searching neighbours among names: '
                    #print v
                    name = v.replace('_','')
                    #print name
                    if not used[name]:
                        #print 'name = ', name
                        if v.endswith('__'):
                             path.insert(0, name)
                        else:
                             path.insert(0, '-' + name)
                        #print 'new path is: ', path
                        inserted_first = True
                        used[name] = True
                        break
                        
                #print 'Searching last gene'   

                if path[-1].startswith('-'):
                    last = path[-1][1:] + '_';
                else:
                    last = path[-1] + '__'
                #print 'Searching gene after ', last
                inserted_last = False
                for v in graph[last]:
                    #print 'Searching neighbours among names: '
                    #print v
                    name = v.replace('_','')
                    #print name
                    if not used[name]:
                        #print 'name = ', name
                        if v.endswith('__'):
                             path.append('-' + name)
                        else:
                             path.append(name)
                        #print 'new path is: ', path
                        inserted_last = True
                        used[name] = True
                        break
                        
                if not inserted_last and not inserted_first:
                	new_path = restore_names(path, ind_names, z_vars)
                	structs_file.write(str(new_path) + '\n')
                	break


def read_indexes(filename):
	ind_name = {}
	with open(filename, 'r') as inds:
		for line in inds.readlines():
			name, index = line.strip().split()
			ind_name[index] = name
	return ind_name
                        
def extract_z_vars(all_vars):
	z_map = {}
	for var in all_vars:
		if var[0].startswith('z') and var[1] == 1:
			v = var[0].replace('_','.').split('.')
			if v[1] not in z_map:
				z_map[v[1]] = {}
			z_map[v[1]][v[2]+'.'+v[3]] = v[2] + '.' + v[4]
	return z_map

if __name__ == "__main__":
	parser = OptionParser()
	parser.add_option('-s', '--sol', dest='xml_solution', help='file with cplex solution')
	parser.add_option('-i', '--index', dest='gene_name_index', help='file indexes of gene names')
	parser.add_option('-o', '--output', dest='structures', help='file with extracted structures')
	(options, args) = parser.parse_args()

	answer, value = parse_xml_answer(options.xml_solution)
	z_vars = extract_z_vars(answer)
	ind_names = read_indexes(options.gene_name_index)
	edges = get_structures(answer)
	with open(options.structures, 'w') as structs_file:
		for key in edges.keys():
			structs_file.write(key + ': ')
			if key not in z_vars:
				z = {}
			else:
				z = z_vars[key]
			build_path(edges_to_graph(edges[key]), ind_names, structs_file, z)
