import re
import xml.etree.ElementTree as ET
from optparse import OptionParser

evaluation_errors = []

def insert_multiplication_sign(line):
    return re.sub('(\d)(\s)(\d)', lambda m: m.group(1) + " * " + m.group(3), line)

def parse_xml_answer(file_name):
    tree = ET.parse(file_name)
    answer = []
    root = tree.getroot()
    for var in root[3]:
        answer.append((var.attrib['name'], int(round(float(var.attrib['value'])))))
    value = root[0].attrib['objectiveValue']
    return answer, value

def replace_equality_sign(line):
    return re.sub('(\s)(=)(\s)', lambda m: m.group(1) + " == " + m.group(3), line)

def validate_objective(line, answer, value):
    global evaluation_errors
    #print 'Validating objective: ', line,
    for var in answer:
        line = replace_equality_sign(insert_multiplication_sign(line.replace(str(var[0]), str(var[1]))))
    result = eval(line)
    #print ', result=', result, ', expected=', value
    if result != int(value):
    	evaluation_errors.append((line, result, value))
    
def validate_expression(line, answer, value):
    global evaluation_errors
    #print 'Validating expression: ', line,
    for var in answer:
        line = replace_equality_sign(insert_multiplication_sign(line.replace(str(var[0]), str(var[1]))))
    result = eval(line)
    #print ', result=', str(result)
    if not result:
    	evaluation_error.append((line, result))

def check_lp(file_name, answer, value):
    print 'Validating answer...'
    global vars
    
    replacement_array = []
    for var in answer:
        if str(var[0].endswith('__')):
            replacement_array.insert(0, var)
        else:
            replacement_array.append(var)
    
    with open(file_name, 'r') as lp_file:
        for line in lp_file.readlines():
            line= line.strip()
            if not line:
                continue
            elif line in ('General', 'Binary'):
                current_function = lambda x, y, z: 0 
            elif line == 'Minimize':
                current_function = validate_objective
            elif line in ('Subject To', 'Bounds'):
                current_function = validate_expression
            elif line == 'End':
                return
            else:
                current_function(line, replacement_array, value)

if __name__ == "__main__":
	parser = OptionParser()
	parser.add_option('-s', '--sol', dest='xml_solution', help='file with cplex solution')
	parser.add_option('-l', '--lp', dest='lp_file', help='file with cplex task in ibm lp format')
	(options, args) = parser.parse_args()

	answer, value = parse_xml_answer(options.xml_solution)
	check_lp(options.lp_file, answer, value)

	if len(evaluation_errors) > 0:
		print 'Errors found during evaluation: ', len(evaluation_errors)
		for error_entry in evaluation_errors:
			print error_entry
	else:
		print 'No errors found in solution'
