// ipl_generatorGGL: A program for generation of structure reconstruction ILP in ibm clp format.
//
// The algorithm by K.Gorbunov, V.Lyubetsky, R.Gershgorin [http://lab6.iitp.ru/en/ilp_generatorGGL/]
//
// (C)2017, Roman Gershgorin, Institute for Information Transmission Problems RAS
// E-mail: gershr{at}mail.ru
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <iostream>
#include <fstream>
#include <string>
#include <cstdint>
#include <vector>
#include <cassert>
#include <sstream>
#include <set>

#include "tree.h"
#include "structure.h"
#include "task_generation.h"
#include "task_storage.h"
#include "options_parser.h"

void PrintLicenseInfo()
{
	std::cout
		<< "ipl_generatorGGL: A program for generation of structure reconstruction ILP in ibm clp format.\n"
		<< "The algorithm by K.Gorbunov, V.Lyubetsky, R.Gershgorin [http://lab6.iitp.ru/en/ilp_generatorGGL/]\n\n"
		<< "(C)2017, Roman Gershgorin, Institute for Information Transmission Problems RAS\n"
		<< "E-mail: gershr{at}mail.ru\n\n"
		<< "This program is free software: you can redistribute it and/or modify\n"
		<< "it under the terms of the GNU General Public License as published by\n"
		<< "the Free Software Foundation, either version 3 of the License, or\n"
		<< "(at your option) any later version.\n\n"
		<< "This program is distributed in the hope that it will be useful,\n"
		<< "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
		<< "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
		<< "GNU General Public License for more details.\n\n"
		<< "You should have received a copy of the GNU General Public License\n"
		<< "along with this program.  If not, see <http://www.gnu.org/licenses/>.\n";
}

int main(int argc, char* argv[])
{
	try
	{
		PrintLicenseInfo();

		const Files files = ParseOptions(argc, argv);

		std::ifstream in(files.inputFile);
	
		if (!in.is_open())
		{
			throw std::runtime_error("Failed to open file");
		}

		Tree tree = ReadTree(in);
		ParalogsMaxParam param = ReadParalogsMax(in);
		Structures structs;
		NameToIndexStorage nameIndexStorage;
		ReadChromosomeStructures(in, structs, nameIndexStorage);

		PrintNameIndex(std::ofstream(files.mappingFile), nameIndexStorage);

		std::ofstream out(files.outputFile);

		PrinterType print = [&out](const std::string& s) { /*out << "\t" << s << "\n";*/ };

		VariablesStorage vars;
		Storage storage;
		std::cout << "Printing Z variables\n";
		PrintZVariables(storage, print, structs, param, vars);
		std::cout << "Printing X variables\n";
		PrintXVariables(storage, print, structs, param, tree, vars);
		std::cout << "Printing Y variables\n";
		PrintYVariables(storage, print, structs, param, tree, vars);
		std::cout << "Printing T variables\n";
		PrintTVariables(storage, print, structs, param, tree, vars);
		std::cout << "Printing R variables\n";
		PrintRVariables(storage, print, structs, param, tree, vars);
		std::cout << "Printing U variables\n";
		PrintUVariables(storage, print, structs, param, tree, vars);
		std::cout << "Printing U1 variables\n";
		PrintU1Variables(storage, print, structs, param, tree, vars);
		std::cout << "Printing P variables\n";
		PrintPVariables(storage, print, vars);

		vars.PrintData(out, storage);

		return 0;
	}
	catch (const std::exception& e)
	{
		std::cout << "Exception occured: " << e.what() << std::endl;
		std::cout << "Use -h option to see help\n";

	}
}