// ipl_generatorGGL: A program for generation of structure reconstruction ILP in ibm clp format.
//
// The algorithm by K.Gorbunov, V.Lyubetsky, R.Gershgorin [http://lab6.iitp.ru/en/ilp_generatorGGL/]
//
// (C)2017, Roman Gershgorin, Institute for Information Transmission Problems RAS
// E-mail: gershr{at}mail.ru
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef _OPTIONS_PARSER_H_
#define _OPTIONS_PARSER_H_

#include <string>
#include <iostream>
#include <sstream>
#include <boost/program_options.hpp>

struct Files
{
	std::string inputFile;
	std::string outputFile;
	std::string mappingFile;
};

Files ParseOptions(int argc, char* argv[])
{
		boost::program_options::options_description desc("Options");
		desc.add_options()
			("help,h", "Show help")
			("input,i", boost::program_options::value<std::string>(), "File with input data")
			("output,o", boost::program_options::value<std::string>(), "Generated clp task file")
			("mapping,m", boost::program_options::value<std::string>(), "Mapping of gene names to indexes");

		boost::program_options::variables_map variableMap;
		boost::program_options::store(boost::program_options::parse_command_line(argc, argv, desc), variableMap);
		boost::program_options::notify(variableMap);

		if (variableMap.count("help")) {
			std::cerr << "Usage: ilp_generatorGGL -i [file] -o [file]\n";
			std::cerr << desc;
			exit(0);
		}

		Files files;
		files.inputFile = variableMap["input"].as<std::string>();
		files.outputFile = variableMap["output"].as<std::string>();
		files.mappingFile = variableMap["mapping"].as<std::string>();

		return files;
}

#endif // _OPTIONS_PARSER_H_
