// ipl_generatorGGL: A program for generation of structure reconstruction ILP in ibm clp format.
//
// The algorithm by K.Gorbunov, V.Lyubetsky, R.Gershgorin [http://lab6.iitp.ru/en/ilp_generatorGGL/]
//
// (C)2017, Roman Gershgorin, Institute for Information Transmission Problems RAS
// E-mail: gershr{at}mail.ru
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef DATA_H_
#define DATA_H_

#include <vector>
#include <locale>

struct Node
{
	int nodeId;
	int parentRef;
	int leftSonRef;
	int rightSonRef;
	std::string name;

	Node()
		: nodeId(0)
		, parentRef(-1)
		, leftSonRef(-1)
		, rightSonRef(-1)
	{}
};


using Tree = std::vector<Node>;

inline bool IsSpecieNameSymbol(char symbol)
{
	return isalpha(symbol) || (symbol == '_') || isdigit(symbol);
}

inline Tree StringToTree(const std::string& treeStr)
{
	int index = 0;

	Tree tree;
	int treePos = 0;
	tree.push_back(Node());

	while (treeStr[index] != ';')
	{
		if (treeStr[index] == '(')
		{
			tree[treePos].leftSonRef = tree.size();
			tree.push_back(Node());
			tree.back().parentRef = treePos;
			treePos = tree[treePos].leftSonRef;
			index++;
		}
		else if (treeStr[index] == ')')
		{
			treePos = tree[treePos].parentRef;
			index++;
		}
		else if (treeStr[index] == ',')
		{
			tree[tree[treePos].parentRef].rightSonRef = tree.size();
			tree.push_back(Node());
			tree.back().parentRef = tree[treePos].parentRef;
			treePos = tree[tree[treePos].parentRef].rightSonRef;
			index++;
		}
		else if (treeStr[index] == ':')
		{
			++index;
			while (isdigit(treeStr[index]))
			{
				tree[treePos].nodeId = tree[treePos].nodeId * 10 + (treeStr[index++] - '0');
			}
		}
		else if (isalpha(treeStr[index]))
		{
			while (IsSpecieNameSymbol(treeStr[index]))
			{
				tree[treePos].name += treeStr[index++];
			}
		}
	}

	return tree;
}

inline Tree ReadTree(std::istream& input)
{
	std::string tree;
	std::getline(input, tree);
	return StringToTree(tree);
}

#endif // DATA_H_